<?php
/**
 * DBERP 进销存系统
 *
 * ==========================================================================
 * @link      http://www.dberp.net/
 * @copyright 北京珑大钜商科技有限公司，并保留所有权利。
 * @license   http://www.dberp.net/license.html License
 * ==========================================================================
 *
 * @author    静静的风 <baron@loongdom.cn>
 *
 */

namespace Finance\Repository;

use Doctrine\ORM\EntityRepository;
use Doctrine\ORM\QueryBuilder;
use Finance\Entity\Payable;

class PayableRepository extends EntityRepository
{

    /**
     * 应付账款
     * @param array $search
     * @return \Doctrine\ORM\Query
     */
    public function findPayableList(array $search = []): \Doctrine\ORM\Query
    {
        $query = $this->getEntityManager()->createQueryBuilder()
            ->select('a')
            ->from(Payable::class, 'a')
            //->where('a.paymentCode = :code')
            //->setParameter('code', 'payable')
            ->orderBy('a.payableId', 'DESC');

        $query = $this->querySearchData($search, $query);

        return $query->getQuery();
    }

    /**
     * 计算合计金额
     * @param array $search
     * @return float|int|mixed[]|string
     */
    public function findPayableTotalAmount(array $search = [])
    {
        $query = $this->getEntityManager()->createQueryBuilder();
        $query->select('SUM(a.paymentAmount) as totalAmount', 'SUM(a.finishAmount) as finishAmount')
            ->from(Payable::class, 'a');

        $query = $this->querySearchData($search, $query);

        return $query->getQuery()->getArrayResult();
    }

    /**
     * 获取列表信息
     * @param $payableIdArray
     * @param $supplierId
     * @return float|int|mixed|string
     */
    public function findPayableIdList($payableIdArray, $supplierId)
    {
        $query = $this->getEntityManager()->createQueryBuilder();
        $query->select('p')
            ->from(Payable::class, 'p')
            ->where($query->expr()->in('p.payableId', $payableIdArray))
            ->andWhere($query->expr()->eq('p.supplierId', ':supplierId'))->setParameter('supplierId', $supplierId);

        return $query->getQuery()->getResult();
    }

    /**
     * ajax应付账款列表
     * @param $supplierId
     * @return float|int|mixed|string
     */
    public function ajaxFindAllPayable($supplierId)
    {
        $query = $this->getEntityManager()->createQueryBuilder();
        $query->select('p')
            ->from(Payable::class, 'p')
            ->where('p.supplierId = :supplierId')->setParameter('supplierId', $supplierId)
            ->andWhere('p.purchaseInvoiceId = 0')
            ->orderBy('p.payableId', 'DESC');

        return $query->getQuery()->getResult();
    }

    /**
     * 对检索信息进行处理
     * @param $search
     * @param QueryBuilder $queryBuilder
     * @return QueryBuilder
     */
    private function querySearchData($search, QueryBuilder $queryBuilder): QueryBuilder
    {
        if(isset($search['pur_start_amount']) && $search['pur_start_amount'] > 0)   $queryBuilder->andWhere($queryBuilder->expr()->gte('a.paymentAmount', $search['pur_start_amount']));
        if(isset($search['pur_end_amount']) && $search['pur_end_amount'] > 0)       $queryBuilder->andWhere($queryBuilder->expr()->lte('a.paymentAmount', $search['pur_end_amount']));
        if(isset($search['start_amount']) && $search['start_amount'] > 0)           $queryBuilder->andWhere($queryBuilder->expr()->gte('a.finishAmount', $search['start_amount']));
        if(isset($search['end_amount']) && $search['end_amount'] > 0)               $queryBuilder->andWhere($queryBuilder->expr()->lte('a.finishAmount', $search['end_amount']));
        if(!empty($search['p_order_sn']))                                           $queryBuilder->andWhere($queryBuilder->expr()->like('a.pOrderSn', "'%".$search['p_order_sn']."%'"));
        if(!empty($search['supplier_name']))                                        $queryBuilder->andWhere($queryBuilder->expr()->like('a.supplierName', "'%".$search['supplier_name']."%'"));
        if(!empty($search['payment_code']))                                         $queryBuilder->andWhere($queryBuilder->expr()->eq('a.paymentCode', ':paymentCode'))->setParameter('paymentCode', $search['payment_code']);
        if(!empty($search['start_date']))                                           $queryBuilder->andWhere($queryBuilder->expr()->gte('a.addTime', ':startTime'))->setParameter('startTime', strtotime($search['start_date']));
        if(!empty($search['end_date']))                                             $queryBuilder->andWhere($queryBuilder->expr()->lte('a.addTime', ':endTime'))->setParameter('endTime', strtotime($search['end_date']));

        return $queryBuilder;
    }
}