<?php
/**
 * DBERP 进销存系统
 *
 * ==========================================================================
 * @link      http://www.dberp.net/
 * @copyright 北京珑大钜商科技有限公司，并保留所有权利。
 * @license   http://www.dberp.net/license.html License
 * ==========================================================================
 *
 * @author    静静的风 <baron@loongdom.cn>
 *
 */

namespace Purchase\Repository;

use Doctrine\ORM\EntityRepository;
use Doctrine\ORM\QueryBuilder;
use Purchase\Entity\WarehouseOrder;

class WarehouseOrderRepository extends EntityRepository
{
    /**
     * 查询入库订单
     * @param array $search
     * @return \Doctrine\ORM\Query
     */
    public function findWarehouseOrderList(array $search = []): \Doctrine\ORM\Query
    {
        $query = $this->getEntityManager()->createQueryBuilder()
            ->select('w', 'o')
            ->from(WarehouseOrder::class, 'w')
            ->join('w.oneOrder', 'o')
            ->orderBy('w.warehouseOrderId', 'DESC');

        $query = $this->querySearchData($search, $query);

        return $query->getQuery();
    }

    /**
     * 获取最大入库单id
     * @return float|int|mixed|string
     * @throws \Doctrine\ORM\NoResultException
     * @throws \Doctrine\ORM\NonUniqueResultException
     */
    public function getMaxWarehouseOrderId()
    {
        $query = $this->getEntityManager()->createQueryBuilder();
        $query->select('MAX(w.warehouseOrderId) as maxWarehouseOrderId')->from(WarehouseOrder::class, 'w');

        $maxId = $query->getQuery()->getSingleScalarResult();

        return $maxId == null ? 1 : $maxId + 1;
    }

    private function querySearchData($search, QueryBuilder $queryBuilder): QueryBuilder
    {
        if(isset($search['start_amount']) && $search['start_amount'] > 0)   $queryBuilder->andWhere($queryBuilder->expr()->gte('w.warehouseOrderAmount', $search['start_amount']));
        if(isset($search['end_amount']) && $search['end_amount'] > 0)       $queryBuilder->andWhere($queryBuilder->expr()->lte('w.warehouseOrderAmount', $search['end_amount']));
        if(!empty($search['warehouse_order_sn']))                           $queryBuilder->andWhere($queryBuilder->expr()->like('w.warehouseOrderSn', "'%".$search['warehouse_order_sn']."%'"));
        if(!empty($search['supplier_contacts']))                            $queryBuilder->andWhere($queryBuilder->expr()->like('o.supplierContacts', "'%".$search['supplier_contacts']."%'"));
        if(!empty($search['supplier_phone']))                               $queryBuilder->andWhere($queryBuilder->expr()->like('o.supplierPhone', "'%".$search['supplier_phone']."%'"));
        if(isset($search['supplier_id']) && $search['supplier_id'] > 0)     $queryBuilder->andWhere($queryBuilder->expr()->eq('o.supplierId', $search['supplier_id']));
        if(!empty($search['payment_code']))                                 $queryBuilder->andWhere($queryBuilder->expr()->eq('o.paymentCode', ':code'))->setParameter('code', $search['payment_code']);
        if(!empty($search['p_order_state']))                                $queryBuilder->andWhere($queryBuilder->expr()->eq('o.pOrderState', $search['p_order_state']));

        $timeFiled = self::checkOrderTimeType($search);
        if(!empty($search['start_time']))       $queryBuilder->andWhere($queryBuilder->expr()->gte($timeFiled, ':startTime'))->setParameter('startTime', strtotime($search['start_time'].' 00:00:00'));
        if(!empty($search['end_time']))         $queryBuilder->andWhere($queryBuilder->expr()->lte($timeFiled, ':endTime'))->setParameter('endTime', strtotime($search['end_time'].' 23:59:59'));

        return $queryBuilder;
    }

    private static function checkOrderTimeType($search): string
    {
        if (!isset($search['time_type']) || !$search['time_type']) return 'o.warehouseTime';

        $timeFiled = 'o.warehouseTime';
        if ($search['time_type'] == 2) $timeFiled = 'o.waitWarehouseTime';

        return $timeFiled;
    }
}