<?php
/**
 * DBERP 进销存系统
 *
 * ==========================================================================
 * @link      http://www.dberp.net/
 * @copyright 北京珑大钜商科技有限公司，并保留所有权利。
 * @license   http://www.dberp.net/license.html License
 * ==========================================================================
 *
 * @author    静静的风 <baron@loongdom.cn>
 *
 */

namespace Sales\Repository;

use Doctrine\ORM\EntityRepository;
use Doctrine\ORM\QueryBuilder;
use Sales\Entity\SalesOrderGoodsReturn;
use Sales\Entity\SalesOrderReturn;

class SalesOrderReturnRepository extends EntityRepository
{
    /**
     * 销售退货单sql
     * @param array $search
     * @return \Doctrine\ORM\Query
     */
    public function findAllSalesOrderReturn($search = [])
    {
        $query = $this->getEntityManager()->createQueryBuilder()
            ->select('s', 'o')
            ->from(SalesOrderReturn::class, 's')
            ->join('s.oneSalesOrder', 'o')
            ->orderBy('s.salesOrderReturnId', 'DESC');

        if (!empty($search['sales_goods_sn'])) {
            $query->andWhere($query->expr()->in('s.salesOrderReturnId', $this->getEntityManager()->createQueryBuilder()
                ->select('g.salesOrderReturnId')->from(SalesOrderGoodsReturn::class, 'g')
                ->where($query->expr()->like('g.goodsSerialNumberStr', "'%".$search['sales_goods_sn']."%'"))
                ->getDQL()));
        }
        $query = $this->querySearchData($search, $query);

        return $query->getQuery();
    }

    private function querySearchData($search, QueryBuilder $queryBuilder)
    {
        if(isset($search['start_amount']) && $search['start_amount'] > 0)   $queryBuilder->andWhere($queryBuilder->expr()->gte('s.salesOrderReturnAmount', $search['start_amount']));
        if(isset($search['end_amount']) && $search['end_amount'] > 0)       $queryBuilder->andWhere($queryBuilder->expr()->lte('s.salesOrderReturnAmount', $search['end_amount']));
        if(!empty($search['sales_order_sn']))                               $queryBuilder->andWhere($queryBuilder->expr()->like('s.salesOrderSn', "'%".$search['sales_order_sn']."%'"));
        if(!empty($search['send_order_sn']))                                $queryBuilder->andWhere($queryBuilder->expr()->like('s.salesSendOrderSn', "'%".$search['send_order_sn']."%'"));
        if(!empty($search['customer_contacts']))                            $queryBuilder->andWhere($queryBuilder->expr()->like('o.customerContacts', "'%".$search['customer_contacts']."%'"));
        if(!empty($search['customer_phone']))                               $queryBuilder->andWhere($queryBuilder->expr()->like('o.customerPhone', "'%".$search['customer_phone']."%'"));
        if(isset($search['customer_id']) && $search['customer_id'] > 0)     $queryBuilder->andWhere($queryBuilder->expr()->eq('o.customerId', $search['customer_id']));
        if(!empty($search['return_state']))                                 $queryBuilder->andWhere($queryBuilder->expr()->eq('s.returnState', $search['return_state']));

        $timeFiled = self::checkOrderTimeType($search);
        if(!empty($search['start_time']))       $queryBuilder->andWhere($queryBuilder->expr()->gte($timeFiled, ':startTime'))->setParameter('startTime', strtotime($search['start_time'].' 00:00:00'));
        if(!empty($search['end_time']))         $queryBuilder->andWhere($queryBuilder->expr()->lte($timeFiled, ':endTime'))->setParameter('endTime', strtotime($search['end_time'].' 23:59:59'));

        return $queryBuilder;
    }

    private static function checkOrderTimeType($search): string
    {
        if (!isset($search['time_type']) || !$search['time_type']) return 'o.returnFinishTime';

        $timeFiled = 'o.returnFinishTime';
        if ($search['time_type'] == -1) $timeFiled = 'o.returnTime';

        return $timeFiled;
    }
}